# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from kent.utils import Item, parse_envelope


class Test_parse_envelope:
    def test_2_items(self):
        payload = (
            b'{"event_id":"9ec79c33ec9942ab8353589fcb2e04dc","dsn":"https://e12d836b15bb49d7bbf99e64295d995b:@sentry.io/42"}\n'
            b'{"type":"attachment","length":10,"content_type":"text/plain","filename":"hello.txt"}\n'
            b"\xef\xbb\xbfHello\r\n\n"
            b'{"type":"event","length":41,"content_type":"application/json","filename":"application.log"}\n'
            b'{"message":"hello world","level":"error"}\n'
        )

        items = list(parse_envelope(payload))

        assert items == [
            Item(
                envelope_header={
                    "dsn": "https://e12d836b15bb49d7bbf99e64295d995b:@sentry.io/42",
                    "event_id": "9ec79c33ec9942ab8353589fcb2e04dc",
                },
                header={
                    "content_type": "text/plain",
                    "filename": "hello.txt",
                    "length": 10,
                    "type": "attachment",
                },
                body=b"\xef\xbb\xbfHello\r\n",
            ),
            Item(
                envelope_header={
                    "dsn": "https://e12d836b15bb49d7bbf99e64295d995b:@sentry.io/42",
                    "event_id": "9ec79c33ec9942ab8353589fcb2e04dc",
                },
                header={
                    "content_type": "application/json",
                    "filename": "application.log",
                    "length": 41,
                    "type": "event",
                },
                body={
                    "level": "error",
                    "message": "hello world",
                },
            ),
        ]

    def test_2_items_missing_trailing_newline(self):
        payload = (
            b'{"event_id":"9ec79c33ec9942ab8353589fcb2e04dc","dsn":"https://e12d836b15bb49d7bbf99e64295d995b:@sentry.io/42"}\n'
            b'{"type":"attachment","length":10,"content_type":"text/plain","filename":"hello.txt"}\n'
            b"\xef\xbb\xbfHello\r\n\n"
            b'{"type":"event","length":41,"content_type":"application/json","filename":"application.log"}\n'
            b'{"message":"hello world","level":"error"}'
        )

        items = list(parse_envelope(payload))

        assert items == [
            Item(
                envelope_header={
                    "dsn": "https://e12d836b15bb49d7bbf99e64295d995b:@sentry.io/42",
                    "event_id": "9ec79c33ec9942ab8353589fcb2e04dc",
                },
                header={
                    "content_type": "text/plain",
                    "filename": "hello.txt",
                    "length": 10,
                    "type": "attachment",
                },
                body=b"\xef\xbb\xbfHello\r\n",
            ),
            Item(
                envelope_header={
                    "dsn": "https://e12d836b15bb49d7bbf99e64295d995b:@sentry.io/42",
                    "event_id": "9ec79c33ec9942ab8353589fcb2e04dc",
                },
                header={
                    "content_type": "application/json",
                    "filename": "application.log",
                    "length": 41,
                    "type": "event",
                },
                body={
                    "level": "error",
                    "message": "hello world",
                },
            ),
        ]

    def test_2_empty_attachments(self):
        payload = (
            b'{"event_id":"9ec79c33ec9942ab8353589fcb2e04dc"}\n'
            b'{"type":"attachment","length":0}\n'
            b"\n"
            b'{"type":"attachment","length":0}\n'
            b"\n"
        )

        items = list(parse_envelope(payload))

        assert items == [
            Item(
                envelope_header={
                    "event_id": "9ec79c33ec9942ab8353589fcb2e04dc",
                },
                header={
                    "length": 0,
                    "type": "attachment",
                },
                body=b"",
            ),
            Item(
                envelope_header={
                    "event_id": "9ec79c33ec9942ab8353589fcb2e04dc",
                },
                header={
                    "length": 0,
                    "type": "attachment",
                },
                body=b"",
            ),
        ]

    def test_2_empty_attachments_newline_omitted(self):
        payload = (
            b'{"event_id":"9ec79c33ec9942ab8353589fcb2e04dc"}\n'
            b'{"type":"attachment","length":0}\n'
            b"\n"
            b'{"type":"attachment","length":0}\n'
        )
        items = list(parse_envelope(payload))

        assert items == [
            Item(
                envelope_header={
                    "event_id": "9ec79c33ec9942ab8353589fcb2e04dc",
                },
                header={
                    "length": 0,
                    "type": "attachment",
                },
                body=b"",
            ),
            Item(
                envelope_header={
                    "event_id": "9ec79c33ec9942ab8353589fcb2e04dc",
                },
                header={
                    "length": 0,
                    "type": "attachment",
                },
                body=b"",
            ),
        ]

    def test_item_with_implicit_length(self):
        payload = (
            b'{"event_id":"9ec79c33ec9942ab8353589fcb2e04dc"}\n'
            b'{"type":"attachment"}\n'
            b"helloworld\n"
        )
        items = list(parse_envelope(payload))

        assert items == [
            Item(
                envelope_header={
                    "event_id": "9ec79c33ec9942ab8353589fcb2e04dc",
                },
                header={
                    "type": "attachment",
                },
                body=b"helloworld",
            ),
        ]

    def test_item_implicit_length_eof_terminator(self):
        payload = (
            b'{"event_id":"9ec79c33ec9942ab8353589fcb2e04dc"}\n'
            b'{"type":"attachment"}\n'
            b"helloworld"
        )
        items = list(parse_envelope(payload))

        assert items == [
            Item(
                envelope_header={
                    "event_id": "9ec79c33ec9942ab8353589fcb2e04dc",
                },
                header={
                    "type": "attachment",
                },
                body=b"helloworld",
            ),
        ]

    def test_empty_envelope_implicit_length_eof(self):
        payload = (
            b"{}\n"
            b'{"type":"session"}\n'
            b'{"started": "2020-02-07T14:16:00Z","attrs":{"release":"sentry-test@1.0.0"}}'
        )
        items = list(parse_envelope(payload))

        assert items == [
            Item(
                envelope_header={},
                header={
                    "type": "session",
                },
                body={
                    "attrs": {
                        "release": "sentry-test@1.0.0",
                    },
                    "started": "2020-02-07T14:16:00Z",
                },
            ),
        ]



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import sys

import kent.app

from flask import cli


os.environ["FLASK_APP"] = "kent.app"


# Prevent the Flask banner from showing
cli.show_server_banner = lambda *args, **kwargs: True


def maybe_show_banner():
    ctx = cli.cli.make_context(info_name=None, args=sys.argv)
    args = cli.cli.parse_args(ctx, args=sys.argv)
    if args[0] == "run":
        cmd = cli.cli.get_command(ctx, name="run")
        parser = cmd.make_parser(ctx)
        opts, _, _ = parser.parse_args(args[1:])
        port = opts.get("port", 5000)
        host = opts.get("host", "127.0.0.1")

        # Convert any adapter to localhost
        if host == "0.0.0.0":
            host = "localhost"
        elif host == "::":
            host = "::1"

        if ":" in host:
            host = f"[{host}]"

        kent.app.BANNER = f"Listening on http://{host}:{port}/"


def main():
    maybe_show_banner()
    cli.main()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from dataclasses import dataclass
import gzip
import json
import logging
from logging.config import dictConfig
import os
from typing import Optional, Union
import uuid
import zlib

from flask import Flask, request, render_template

from kent import __version__
from kent.utils import parse_envelope


dictConfig(
    {
        "version": 1,
        "formatters": {
            "default": {
                "format": "[%(asctime)s] %(levelname)s: %(name)s %(message)s",
            }
        },
        "handlers": {
            "wsgi": {
                "class": "logging.StreamHandler",
                "stream": "ext://flask.logging.wsgi_errors_stream",
                "formatter": "default",
            },
        },
        "loggers": {
            "kent": {"level": "INFO"},
            "werkzeug": {"level": "ERROR"},
        },
        "root": {
            "level": "INFO",
            "handlers": ["wsgi"],
        },
    }
)


BANNER = None


def deep_get(structure, path, default=None):
    node = structure
    for part in path.split("."):
        if part.startswith("["):
            index = int(part[1:-1])
            node = node[index]
        elif part in node:
            node = node[part]
        else:
            return default
    return node


@dataclass
class Event:
    project_id: int
    event_id: str

    # envelope_header when the envelope API is used
    envelope_header: Optional[dict] = None
    # item header
    header: Optional[dict] = None
    # item
    # attachments will be stored as bytes, non-attachments as python
    # datastructures
    body: Optional[Union[dict, bytes]] = None

    @property
    def summary(self):
        if not self.body:
            return "no summary"

        if isinstance(self.body, dict):
            # Kent body parsing errors
            kent_error = self.body.get("error")
            if kent_error:
                return kent_error

            # Sentry exceptions events
            exceptions = deep_get(self.body, "exception.values", default=[])
            if exceptions:
                first = exceptions[0]
                return f"{first['type']}: {first['value']}"

            # Sentry message
            msg = deep_get(self.body, "message", default=None)
            if msg:
                return msg

            # CSP security report (older browsers)
            if "csp-report" in self.body:
                directive = deep_get(
                    self.body, "csp-report.violated-directive", default="unknown"
                )
                summary = f"csp-report: {directive}"
                return summary

        elif isinstance(self.body, list):
            # CSP security report (newer browsers)
            if self.body[0].get("type") == "csp-violation":
                directives = []
                for section in self.body[0]:
                    directives.append(section.get("effectiveDirective") or "unknown")

                all_directives = ", ".join(directives)
                summary = f"csp-report: {all_directives}"
                return summary

        return "no summary"

    @property
    def timestamp(self):
        # NOTE(willkg): timestamp is a string
        return isinstance(self.body, dict) and self.body.get("timestamp") or "none"

    def to_dict(self):
        return {
            "project_id": self.project_id,
            "event_id": self.event_id,
            "payload": {
                "envelope_header": self.envelope_header,
                "header": self.header,
                "body": self.body,
            },
        }


class EventManager:
    MAX_EVENTS = 100

    def __init__(self):
        # List of Event instances
        self.events = []

    def add_event(
        self, event_id, project_id, envelope_header=None, header=None, body=None
    ):
        event = Event(
            project_id=project_id,
            event_id=event_id,
            envelope_header=envelope_header,
            header=header,
            body=body,
        )
        self.events.append(event)

        while len(self.events) > self.MAX_EVENTS:
            self.events.pop(0)

    def get_event(self, event_id):
        for event in self.events:
            if event.event_id == event_id:
                return event
        return None

    def get_events(self):
        return self.events

    def flush(self):
        self.events = []


EVENTS = EventManager()


INTERESTING_HEADERS = [
    "User-Agent",
    "X-Sentry-Auth",
]


def create_app(test_config=None):
    dev_mode = os.environ.get("KENT_DEV", "0") == "1"

    # Always start an app with an empty error manager
    EVENTS.flush()

    app = Flask(__name__, instance_relative_config=True)
    app.config.from_mapping(SECRET_KEY="dev")

    if test_config is not None:
        app.config.from_mapping(test_config)

    if BANNER:
        app.logger.info(BANNER)

    if dev_mode:
        logging.getLogger("kent").setLevel(logging.DEBUG)
        app.logger.debug("Dev mode on.")

    @app.route("/", methods=["GET"])
    def index_view():
        host = request.scheme + "://" + request.headers["host"]
        dsn = request.scheme + "://public@" + request.headers["host"] + "/1"

        return render_template(
            "index.html",
            host=host,
            dsn=dsn,
            events=EVENTS.get_events(),
            version=__version__,
        )

    @app.route("/api/event/<event_id>", methods=["GET"])
    def api_event_view(event_id):
        app.logger.info(f"GET /api/event/{event_id}")
        event = EVENTS.get_event(event_id)
        if event is None:
            return {"error": f"Event {event_id} not found"}, 404

        return event.to_dict()

    @app.route("/api/eventlist/", methods=["GET"])
    def api_event_list_view():
        app.logger.info("GET /api/eventlist/")
        event_ids = [
            {
                "project_id": event.project_id,
                "event_id": event.event_id,
                "summary": event.summary,
            }
            for event in EVENTS.get_events()
        ]
        return {"events": event_ids}

    @app.route("/api/flush/", methods=["POST"])
    def api_flush_view():
        app.logger.info("POST /api/flush")
        EVENTS.flush()
        return {"success": True}

    def log_headers(dev_mode, error_id, headers):
        # Log headers
        if dev_mode:
            for key, val in headers.items():
                app.logger.info("%s: header: %s: %s", error_id, key, val)
        else:
            for key in INTERESTING_HEADERS:
                if key in headers:
                    app.logger.info(
                        "%s: header: %s: %s", error_id, key, request.headers[key]
                    )

    @app.route("/api/<int:project_id>/store/", methods=["POST"])
    def store_view(project_id):
        app.logger.info(f"POST /api/{project_id}/store/")
        event_id = str(uuid.uuid4())
        log_headers(dev_mode, event_id, request.headers)

        # Decompress it
        if request.headers.get("content-encoding") == "gzip":
            body = gzip.decompress(request.data)
        elif request.headers.get("content-encoding") == "deflate":
            body = zlib.decompress(request.data)
        else:
            body = request.data

        app.logger.debug(f"{body}")

        # JSON decode it
        try:
            json_body = json.loads(body)
        except Exception:
            app.logger.exception("%s: exception when JSON-decoding body.", event_id)
            app.logger.error("%s: %s", event_id, json_body)
            EVENTS.add_event(
                event_id=event_id,
                project_id=project_id,
                body={"error": "Kent could not decode body; see logs"},
            )
            raise

        EVENTS.add_event(event_id=event_id, project_id=project_id, body=json_body)

        # Log interesting bits in the body
        if "exception" in json_body:
            app.logger.info(
                "%s: exception: %s %s",
                event_id,
                deep_get(json_body, "exception.values.[0].type"),
                deep_get(json_body, "exception.values.[0].value"),
            )
        if "message" in json_body:
            app.logger.info("%s: message: %s", event_id, deep_get(json_body, "message"))
        app.logger.info(
            "%s: sdk: %s %s",
            event_id,
            deep_get(json_body, "sdk.name"),
            deep_get(json_body, "sdk.version"),
        )

        # Log event url
        event_url = f"{request.scheme}://{request.headers['host']}/api/event/{event_id}"
        app.logger.info("%s: project id: %s", event_id, project_id)
        app.logger.info("%s: url: %s", event_id, event_url)

        return {"success": True}

    @app.route("/api/<int:project_id>/envelope/", methods=["POST"])
    def envelope_view(project_id):
        app.logger.info(f"POST /api/{project_id}/envelope/")
        request_id = str(uuid.uuid4())
        log_headers(dev_mode, request_id, request.headers)

        # Decompress it
        if request.headers.get("content-encoding") == "gzip":
            body = gzip.decompress(request.data)
        elif request.headers.get("content-encoding") == "deflate":
            body = zlib.decompress(request.data)
        else:
            body = request.data

        app.logger.debug(f"{body}")

        for item in parse_envelope(body):
            event_id = str(uuid.uuid4())
            EVENTS.add_event(
                event_id=event_id,
                project_id=project_id,
                envelope_header=item.envelope_header,
                header=item.header,
                body=item.body,
            )

            # Log interesting bits in the body
            if "exception" in item.body:
                app.logger.info(
                    "%s: exception: %s %s",
                    event_id,
                    deep_get(item.body, "exception.values.[0].type"),
                    deep_get(item.body, "exception.values.[0].value"),
                )
            if "message" in item.body:
                app.logger.info(
                    "%s: message: %s", event_id, deep_get(item.body, "message")
                )
            app.logger.info(
                "%s: sdk: %s %s",
                event_id,
                deep_get(item.body, "sdk.name"),
                deep_get(item.body, "sdk.version"),
            )

            # Log event url
            event_url = (
                f"{request.scheme}://{request.headers['host']}/api/event/{event_id}"
            )
            app.logger.info("%s: project id: %s", event_id, project_id)
            app.logger.info("%s: url: %s", event_id, event_url)

        return {"success": True}

    @app.route("/api/<int:project_id>/security/", methods=["POST"])
    def security_view(project_id):
        app.logger.info(f"POST /api/{project_id}/security/")
        event_id = str(uuid.uuid4())
        log_headers(dev_mode, event_id, request.headers)

        body = request.data

        app.logger.debug(f"{body}")

        # Decode the JSON payload
        try:
            json_body = json.loads(body)
        except Exception:
            app.logger.exception("%s: exception when JSON-decoding body.", event_id)
            app.logger.error("%s: %s", event_id, body)
            EVENTS.add_event(
                event_id=event_id,
                project_id=project_id,
                body={"error": "Kent could not decode body; see logs"},
            )
            raise

        EVENTS.add_event(event_id=event_id, project_id=project_id, body=json_body)

        # Log interesting bits in the body
        for i, report in enumerate(json_body):
            if "type" in report:
                app.logger.info(
                    "%s: %s: type: %s",
                    event_id,
                    i,
                    report["type"],
                )
        app.logger.info(
            "%s: project id: %s",
            event_id,
            project_id,
        )

        # Log event url
        event_url = f"{request.scheme}://{request.headers['host']}/api/event/{event_id}"
        app.logger.info(
            "%s: url: %s",
            event_id,
            event_url,
        )

        return {"success": True}

    return app



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from dataclasses import dataclass
import logging
import json
from typing import Union


LOGGER = logging.getLogger(__name__)


@dataclass
class Item:
    envelope_header: dict
    header: dict
    body: Union[dict, bytes]


def get_newline_index(body, start_index, end_index):
    end_index = body.find(b"\n", start_index)
    if end_index == -1:
        # If there are no more \n, then the end_index is the last index in the
        # body
        end_index = len(body)
    else:
        while body[end_index] == "\r":
            end_index = body.find(b"\n", end_index + 1)
    return end_index


def parse_envelope(body):
    """Parses an envelope payload into items

    :arg body: the envelope payload body

    :returns: generator of items

    """

    body_length = len(body)
    start_index = end_index = 0
    read_length = -1

    envelope_header = None

    # Absorb envelope
    # See: https://develop.sentry.dev/sdk/envelopes/
    while end_index < body_length:
        start_index = end_index
        end_index = get_newline_index(body, start_index, end_index)

        if envelope_header is None:
            envelope_header = json.loads(body[start_index:end_index])
            end_index += 1
            continue

        json_part = body[start_index:end_index]

        try:
            part = json.loads(json_part)
        except Exception:
            LOGGER.exception("exception when JSON-decoding body.")
            LOGGER.error("%s", json_part)
            raise

        if "type" in part:
            # Advance past the \n
            end_index += 1

            start_index = end_index
            read_length = part.get("length", -1)
            if read_length != -1:
                # NOTE(willkg): This will include the newline separater at the end
                end_index = end_index + read_length
            else:
                end_index = get_newline_index(body, start_index, end_index)

            # NOTE(willkg): This drops the newline separator because it's not
            # part of the Item body
            item_body = body[start_index:end_index]

            if part.get("type") == "attachment":
                yield Item(
                    envelope_header=envelope_header,
                    header=part,
                    body=item_body,
                )

            else:
                item_body_data = json.loads(item_body)
                yield Item(
                    envelope_header=envelope_header, header=part, body=item_body_data
                )

            # Advance past the \n
            end_index += 1
            continue



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import argparse
from importlib.metadata import version as metadata_version
import logging
from urllib.parse import urlparse
import sys

try:
    import requests
except ImportError:
    print("You need to have requests installed.")
    sys.exit(1)

try:
    from sentry_sdk import init, capture_exception, capture_message
except ImportError:
    print("You need to have sentry_sdk installed.")
    sys.exit(1)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--dsn", default="http://public@localhost:5000/1", help="SENTRY_DSN to use"
    )
    parser.add_argument(
        "kind",
        nargs="?",
        default="message",
        help=(
            "What kind of thing to post. ['message', 'error', 'loggingerror', "
            + "'security_csp_new', 'security_csp_old']"
        ),
    )

    args = parser.parse_args()

    logging.basicConfig(level=logging.ERROR)
    init(args.dsn)

    print(f"DSN: {args.dsn}")
    print(f"Sentry-sdk version: {metadata_version('sentry_sdk')}")
    if args.kind == "message":
        capture_message("test error capture")
        print(f"Message posted to: {args.dsn}")
        return

    elif args.kind == "error":
        try:
            raise Exception("intentional exception")
        except Exception as exc:
            capture_exception(exc)
        print(f"Error posted to: {args.dsn}")
        return

    elif args.kind == "loggingerror":
        try:
            raise Exception("intentional exception")
        except Exception:
            logging.exception("intentional exception")
        print(f"Error posted to: {args.dsn}")
        return

    elif args.kind.startswith("security"):
        parsed = urlparse(args.dsn)
        report_uri = f"{parsed.scheme}://{parsed.username}@{parsed.netloc}/api{parsed.path}/security/"

        data = None

        if args.kind == "security_csp_new":
            # Newer browsers send this structure
            # From: https://w3c.github.io/reporting/#sample-reports
            data = [
                {
                    "age": 0,
                    "body": {
                        "blockedURL": "https://maps.googleapis.com/maps/api/js",
                        "disposition": "enforce",
                        "documentURL": "https://test.example.com/",
                        "effectiveDirective": "script-src",
                        "originalPolicy": (
                            "default-src 'self'; "
                            + "img-src 'self'; "
                            + "script-src 'self'; "
                            + "form-action 'self'; "
                            + "frame-ancestors 'self'; "
                            + "report-to csp-endpoint; "
                            + f"report-uri {report_uri}"
                        ),
                        "referrer": "",
                        "statusCode": 200,
                    },
                    "type": "csp-violation",
                    "url": "https://test.example.com/",
                    "user_agent": "Mozilla/5.0 (user agent)",
                }
            ]

        elif args.kind == "security_csp_old":
            # Older browsers send this structure
            data = {
                "csp-report": {
                    "blocked-uri": "https://www.youtube.com",
                    "disposition": "enforce",
                    "document-uri": "http://localhost:8000/test-page/",
                    "effective-directive": "frame-src",
                    "original-policy": (
                        "style-src 'self' 'unsafe-inline'; "
                        + "base-uri 'none'; "
                        + "media-src 'self'; "
                        + "object-src 'none'; "
                        + "child-src 'self'; "
                        + "img-src 'self' data: http://test.example.com/; "
                        + "font-src 'self'; "
                        + "default-src 'self'; "
                        + "connect-src 'self' http://test.example.com/; "
                        + "script-src 'self' http://test.example.com/; "
                        + "frame-src 'self'; "
                        + f"report-uri {report_uri}"
                    ),
                    "referrer": "",
                    "status-code": 0,
                    "violated-directive": "frame-src",
                }
            }

        if data:
            resp = requests.post(report_uri, json=data)
            resp.raise_for_status()
            print(f"Security report posted to: {report_uri}")
            return

    print(f"{args.kind!r} is not a valid kind.")
    return 1


if __name__ == "__main__":
    sys.exit(main())



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import pytest
import uuid

from kent.app import create_app, Event


@pytest.fixture
def client():
    app = create_app({"TESTING": True})

    with app.test_client() as client:
        yield client


class TestEvent:
    @pytest.mark.parametrize(
        "payload, expected",
        [
            # Empty event payload
            ({}, "no summary"),
            # Event payload for an exception
            (
                {
                    "exception": {
                        "values": [
                            {"type": "Exception", "value": "Intentional exception"}
                        ]
                    }
                },
                "Exception: Intentional exception",
            ),
            # Message payload
            (
                {
                    "message": "some message",
                },
                "some message",
            ),
        ],
    )
    def test_summary(self, payload, expected):
        event = Event(
            project_id="0",
            event_id="9884b351-1e8f-4a28-8a9a-fc0033467e4e",
            body=payload,
        )
        assert event.summary == expected


def test_index_view(client):
    resp = client.get("/")
    assert b"Kent" in resp.data


# From "kent-testpost" message with Python sentry-sdk 1.45.0
SENTRY_SDK_1_45_0_ERROR = {
    "breadcrumbs": {"values": []},
    "contexts": {
        "runtime": {
            "build": "3.10.14 (main, May  6 2024, 10:26:19) [GCC 13.2.0]",
            "name": "CPython",
            "version": "3.10.14",
        },
        "trace": {
            "parent_span_id": None,
            "span_id": "93e11289cc659ab8",
            "trace_id": "b7c12e7267ff4886a935914889833319",
        },
    },
    "environment": "production",
    "event_id": "9ae282ae03b6476ebeb1392a151edf55",
    "extra": {"sys.argv": ["/home/willkg/venvs/kent/bin/kent-testpost", "message"]},
    "level": "info",
    "message": "test error capture",
    "modules": {
        "blinker": "1.8.2",
        "certifi": "2024.2.2",
        "charset-normalizer": "3.3.2",
        "click": "8.1.7",
        "flask": "3.0.3",
        "idna": "3.7",
        "itsdangerous": "2.2.0",
        "jinja2": "3.1.4",
        "kent": "1.2.0",
        "markupsafe": "2.1.5",
        "pip": "24.0",
        "requests": "2.31.0",
        "sentry-sdk": "1.45.0",
        "setuptools": "69.5.1",
        "urllib3": "2.2.1",
        "werkzeug": "3.0.3",
        "wheel": "0.43.0",
    },
    "platform": "python",
    "release": "2667f04c7246db93ecbc175cf7a2534ad412154e",
    "sdk": {
        "integrations": [
            "argv",
            "atexit",
            "dedupe",
            "excepthook",
            "flask",
            "logging",
            "modules",
            "stdlib",
            "threading",
        ],
        "name": "sentry.python.flask",
        "packages": [{"name": "pypi:sentry-sdk", "version": "1.45.0"}],
        "version": "1.45.0",
    },
    "server_name": "saturn7",
    "timestamp": "2024-05-19T02:22:32.086845Z",
    "transaction_info": {},
}


def test_store_view(client):
    resp = client.post(
        "/api/1/store/",
        json=SENTRY_SDK_1_45_0_ERROR,
    )

    assert resp.status_code == 200


def test_envelope_view(client):
    resp = client.post(
        "/api/1/envelope/",
        content_type="application/octet-stream",
        # From "kent-testpost message" with Python sentry-sdk 2.2.0
        data=(
            b'{"event_id":"b5a2369b82c7421eb5c118a3151da03e","sent_at":"2024-05-19T03:13:43.016351Z","trace":{"trace_id":"e2a6aebaa043485e93cb5c8df6fbc230","environment":"production","release":"2667f04c7246db93ecbc175cf7a2534ad412154e","public_key":"public"}}\n{"type":"event","content_type":"application/json","length":1164}\n{"message":"test error capture","level":"info","event_id":"b5a2369b82c7421eb5c118a3151da03e","timestamp":"2024-05-19T03:13:43.015857Z","contexts":{"trace":{"trace_id":"e2a6aebaa043485e93cb5c8df6fbc230","span_id":"aa35c2671f716d4d","parent_span_id":null},"runtime":{"name":"CPython","version":"3.10.14","build":"3.10.14 (main, May  6 2024, 10:26:19) [GCC 13.2.0]"}},"transaction_info":{},"breadcrumbs":{"values":[]},"extra":{"sys.argv":["/home/willkg/venvs/kent/bin/kent-testpost","message"]},"modules":{"pip":"24.0","flask":"3.0.3","sentry-sdk":"2.2.0","wheel":"0.43.0","idna":"3.7","setuptools":"69.5.1","click":"8.1.7","blinker":"1.8.2","urllib3":"2.2.1","kent":"1.2.0","markupsafe":"2.1.5","werkzeug":"3.0.3","charset-normalizer":"3.3.2","itsdangerous":"2.2.0","requests":"2.31.0","jinja2":"3.1.4","certifi":"2024.2.2"},"release":"2667f04c7246db93ecbc175cf7a2534ad412154e","environment":"production","server_name":"saturn7","sdk":{"name":"sentry.python.flask","version":"2.2.0","packages":[{"name":"pypi:sentry-sdk","version":"2.2.0"}],"integrations":["argv","atexit","dedupe","excepthook","flask","logging","modules","stdlib","threading"]},"platform":"python"}\n'
        ),
    )
    assert resp.status_code == 200

    # Get all the events
    resp = client.get("/api/eventlist/")
    assert resp.status_code == 200
    event_id = resp.json["events"][0]["event_id"]

    # Get the event by event_id
    resp = client.get(f"/api/event/{event_id}")
    assert resp.status_code == 200
    assert resp.json == {
        "event_id": event_id,
        "payload": {
            "body": {
                "breadcrumbs": {
                    "values": [],
                },
                "contexts": {
                    "runtime": {
                        "build": "3.10.14 (main, May  6 2024, 10:26:19) [GCC 13.2.0]",
                        "name": "CPython",
                        "version": "3.10.14",
                    },
                    "trace": {
                        "parent_span_id": None,
                        "span_id": "aa35c2671f716d4d",
                        "trace_id": "e2a6aebaa043485e93cb5c8df6fbc230",
                    },
                },
                "environment": "production",
                "event_id": "b5a2369b82c7421eb5c118a3151da03e",
                "extra": {
                    "sys.argv": [
                        "/home/willkg/venvs/kent/bin/kent-testpost",
                        "message",
                    ],
                },
                "level": "info",
                "message": "test error capture",
                "modules": {
                    "blinker": "1.8.2",
                    "certifi": "2024.2.2",
                    "charset-normalizer": "3.3.2",
                    "click": "8.1.7",
                    "flask": "3.0.3",
                    "idna": "3.7",
                    "itsdangerous": "2.2.0",
                    "jinja2": "3.1.4",
                    "kent": "1.2.0",
                    "markupsafe": "2.1.5",
                    "pip": "24.0",
                    "requests": "2.31.0",
                    "sentry-sdk": "2.2.0",
                    "setuptools": "69.5.1",
                    "urllib3": "2.2.1",
                    "werkzeug": "3.0.3",
                    "wheel": "0.43.0",
                },
                "platform": "python",
                "release": "2667f04c7246db93ecbc175cf7a2534ad412154e",
                "sdk": {
                    "integrations": [
                        "argv",
                        "atexit",
                        "dedupe",
                        "excepthook",
                        "flask",
                        "logging",
                        "modules",
                        "stdlib",
                        "threading",
                    ],
                    "name": "sentry.python.flask",
                    "packages": [
                        {
                            "name": "pypi:sentry-sdk",
                            "version": "2.2.0",
                        },
                    ],
                    "version": "2.2.0",
                },
                "server_name": "saturn7",
                "timestamp": "2024-05-19T03:13:43.015857Z",
                "transaction_info": {},
            },
            "envelope_header": {
                "event_id": "b5a2369b82c7421eb5c118a3151da03e",
                "sent_at": "2024-05-19T03:13:43.016351Z",
                "trace": {
                    "environment": "production",
                    "public_key": "public",
                    "release": "2667f04c7246db93ecbc175cf7a2534ad412154e",
                    "trace_id": "e2a6aebaa043485e93cb5c8df6fbc230",
                },
            },
            "header": {
                "content_type": "application/json",
                "length": 1164,
                "type": "event",
            },
        },
        "project_id": 1,
    }


# From "kent-testpost security_csp_new"
CSP_REPORT_NEW = [
    {
        "age": 0,
        "body": {
            "blockedURL": "https://maps.googleapis.com/maps/api/js",
            "disposition": "enforce",
            "documentURL": "https://test.example.com/",
            "effectiveDirective": "script-src",
            "originalPolicy": "default-src 'self'; img-src 'self'; script-src 'self'; form-action 'self'; frame-ancestors 'self'; report-to csp-endpoint; report-uri http://public@public@localhost:5000/api/1/security/",
            "referrer": "",
            "statusCode": 200,
        },
        "type": "csp-violation",
        "url": "https://test.example.com/",
        "user_agent": "Mozilla/5.0 (user agent)",
    }
]


def test_security_view(client):
    resp = client.post(
        "/api/1/security/",
        # From "kent-testpost security_csp_new"
        json=CSP_REPORT_NEW,
    )
    assert resp.status_code == 200


def test_api_flush_view(client):
    resp = client.post("/api/flush/")
    assert resp.status_code == 200
    assert resp.json == {"success": True}


class TestAPIEventListView:
    def test_empty_eventlist(self, client):
        resp = client.get("/api/eventlist/")
        assert resp.json == {"events": []}

    def test_nonempty_eventlist(self, client):
        # Store an event
        resp = client.post(
            "/api/1/store/",
            # From "kent-testpost" message with Python sentry-sdk 1.45.0
            json=SENTRY_SDK_1_45_0_ERROR,
        )

        assert resp.status_code == 200

        resp = client.get("/api/eventlist/")
        assert len(resp.json) == 1


class TestAPIEventView:
    def test_404(self, client):
        event_id = str(uuid.uuid4())
        resp = client.get(f"/api/event/{event_id}")
        assert resp.status_code == 404
        assert resp.json == {"error": f"Event {event_id} not found"}

    def test_event_exists(self, client):
        # Store an event
        resp = client.post("/api/1/store/", json=SENTRY_SDK_1_45_0_ERROR)
        assert resp.status_code == 200

        # Get all the events
        resp = client.get("/api/eventlist/")
        assert resp.status_code == 200
        event_id = resp.json["events"][0]["event_id"]

        # Get the event by event_id
        resp = client.get(f"/api/event/{event_id}")
        assert resp.status_code == 200
        assert resp.json == {
            "project_id": 1,
            "event_id": event_id,
            "payload": {
                "envelope_header": None,
                "header": None,
                "body": SENTRY_SDK_1_45_0_ERROR,
            },
        }



#!/usr/bin/env python

# Usage: python bin/kent_submit.py [FILE]

import argparse
import json

import requests


def main():
    parser = argparse.ArgumentParser(description="submit sentry events to kent")
    parser.add_argument("--host", default="localhost")
    parser.add_argument("--port", default="8000")
    parser.add_argument("file", nargs="+", help="entry files to submit")

    args = parser.parse_args()

    for fn in args.file:
        with open(fn, "r") as fp:
            data = json.load(fp)

            if "payload" in data:
                data = data["payload"]

            print(f"Submitting {fn}...")
            resp = requests.post(
                f"http://{args.host}:{args.port}/api/0/store/", json=data
            )
            resp.raise_for_status()


if __name__ == "__main__":
    main()
